@@ -23,8 +23,13 @@ use crate::{
     circuit_optimizer::{OptimizationContext, OptimizationSettings},
 };
 
-#[pyfunction]
-pub fn batch_to_concat(circuit: CircuitRc, axis: usize, num_batches: usize) -> Result<CircuitRc> {
+#[pyfunction(keep_non_axis_leaves = "false")]
+pub fn batch_to_concat(
+    circuit: CircuitRc,
+    axis: usize,
+    num_batches: usize,
+    keep_non_axis_leaves: bool, // THIS IS A HACK
+) -> Result<CircuitRc> {
     let l = circuit.info().shape[axis];
     if l % num_batches != 0 {
         bail!(BatchError::NumberDoesntDivide { l, num_batches });
@@ -37,6 +42,11 @@ pub fn batch_to_concat(circuit: CircuitRc, axis: usize, num_batches: usize) -> R
     });
 
     let (name, circuit, leaves_axis, leaves_non_axis) = get_axis_leaves(circuit, axis)?;
+    let leaves_non_axis = if keep_non_axis_leaves {
+        Default::default()
+    } else {
+        leaves_non_axis
+    };
     // println!("leaves axis");
     // for (k, v) in &leaves_axis {
     //     println!("{}", v);
@@ -273,7 +283,13 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> Resul
     let batchheads_from: Vec<CircuitRc> = batchheads
         .iter()
         .map(|c| {
-            batch_to_concat(c.clone(), axis_of_name(c, &batch_str).unwrap(), num_batches).unwrap()
+            batch_to_concat(
+                c.clone(),
+                axis_of_name(c, &batch_str).unwrap(),
+                num_batches,
+                false,
+            )
+            .unwrap()
         })
         .collect();
     let out_still_symbols = deep_map_op(nested_big.crc(), |c| {
